/*       ROBOT               Ignacio  Merino  Lagunilla                       */

#include <unistd.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <netdb.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "global.hpp"

//#define 	DEBUG


robot::robot(void)
{
	v1 = v2 = 0;
	dibu_sen = dibu_rastro = FALSO;
	activo = CIERTO;
	accesible = CIERTO;
	ini_s = fin_s = NULL;
	sig = ant = NULL;
}

robot::~robot(void)
{
#ifdef DEBUG
	printf("Destruyo el proceso robot\n");
	if (fichero == CIERTO) fprintf(accion , "Destruyo el proceso robot\n");
#endif
	if(ini_s != NULL) delete(ini_s);
	if(sig != NULL) delete(sig);
}

int robot::crear_primero(void)
{
 struct sockaddr_in dir;

	fin_r = this;
	if( (sock = socket(AF_INET , SOCK_STREAM , 0)) < 0)
	{
		printf("\aERROR: no se puede crear el socket principal del proceso robot.\n");
		if(fichero == CIERTO) fprintf(accion , "ERROR: no se puede crear el socket principal del proceso robot.\n");
		return(-1);
	}
#ifdef DEBUG
	printf("Creo el socket principal del proceso robot\n");
	if(fichero == CIERTO) fprintf(accion , "Creo el socket principal del proceso robot\n");
#endif
	bzero((char *) &dir , sizeof(struct sockaddr_in));
	dir.sin_family = AF_INET;
	dir.sin_addr.s_addr = htonl(INADDR_ANY);
	dir.sin_port = htons(puerto_r);
	if(bind(sock , (struct sockaddr*) &dir , sizeof(dir)) < 0)
	{
		printf("\aERROR: no se puede asociar la direccion al socket principal del proceso robot.\n");
        	if(fichero == CIERTO) fprintf(accion , "ERROR: no se puede asociar la direccion al socket principal del proceso robot.\n");
		close(sock);
		return(-2);
	}
#ifdef DEBUG
	printf("Asocio el socket principal del proceso robot a su direccion\n");
	if(fichero == CIERTO) fprintf(accion , "Asocio el socket principal del proceso robot a su direccion\n");
#endif
	if(listen(sock , 5))
	{
		printf("\aERROR: No se puede escuchar por el socket principal del proceso robot.\n");
		if(fichero == CIERTO) fprintf(accion , "ERROR: No se puede escuchar por el socket principal del proceso robot.\n");
		close(sock);
		return(-3);
	}
#ifdef DEBUG
	printf("Escucho por el socket principal del proceso robot\n");
	if(fichero == CIERTO) fprintf(accion , "Escucho por el socket principal del proceso robot\n");
#endif
	return(0);
}

int robot::crear(void)
{
 struct sockaddr_in dir;
 int longitud = sizeof(struct sockaddr_in) , puerto;

	if( (sock = socket(AF_INET , SOCK_STREAM , 0)) < 0)
	{
		printf("\aERROR: No se puede crear el socket del proceso robot.\n");
		if(fichero == CIERTO) fprintf(accion , "ERROR: No se puede crear el socket del proceso robot.\n");
		return(-1);
	}
#ifdef DEBUG
	printf("Creo un socket para un proceso robot\n");
	if(fichero == CIERTO) fprintf(accion , "Creo un socket para un proceso robot\n");
#endif
	bzero((char *) &dir , longitud);
	dir.sin_family = AF_INET;
	dir.sin_addr.s_addr = htonl(INADDR_ANY);
	dir.sin_port = htons(0);
	if(bind(sock , (struct sockaddr *) &dir , sizeof(dir)) < 0)
	{
        	printf("\aERROR: no se puede asociar la direccion al socket del proceso robot.\n");
        	if(fichero == CIERTO) fprintf(accion , "ERROR: no se puede asociar la direccion al socket del proceso robot.\n");
		close(sock);
		return(-2);
	}
#ifdef DEBUG
	printf("Asocio el socket del proceso robot a su direccion\n");
	if(fichero == CIERTO) fprintf(accion , "Asocio el socket del proceso robot a su direccion\n");
#endif
	if(listen(sock , 1))
	{
		printf("\aERROR: No se puede escuchar por el socket del proceso robot.\n");
		if(fichero == CIERTO) fprintf(accion , "ERROR: No se puede escuchar por el socket del proceso robot.\n");
		close(sock);
		return(-3);
	}
#ifdef DEBUG
	printf("Escucho por el socket del proceso robot\n");
	if(fichero == CIERTO) fprintf(accion , "Escucho por el socket del proceso robot\n");
#endif
	if(getsockname(sock , (struct sockaddr *) &dir , &longitud) < 0)
	{
		printf("\aERROR: no se puede acceder a los datos de socket.\n");
		if(fichero == CIERTO) fprintf(accion , "ERROR: no se puede acceder a los datos de socket.\n");
        	close(sock);
		return(-4);
	}
	puerto = htons(dir.sin_port);
#ifdef DEBUG
	printf("Obtengo el puerto %d para el proceso robot\n" , puerto);
	if(fichero == CIERTO) fprintf(accion , "Obtengo el puerto %d para el proceso robot\n" , puerto);
#endif
	ant = fin_r;
	fin_r->sig = this;
	fin_r = this;
#ifdef DEBUG
	printf("Enlazo el proceso robot a la lista de robots\n");
	if (fichero == CIERTO) fprintf(accion , "Enlazo el proceso robot a la lista de robots\n");
#endif
	return(puerto);
}

void robot::destruir(int cerrar , int dibujar)
{
 float x , y , s , c , s1 , c1;
 robot *fin , *aux = ini_r->sig;

#ifdef DEBUG
	printf("Destruyo el proceso robot '%s'\n" , nombre);
	if(fichero == CIERTO) fprintf(accion , "Destruyo el proceso robot '%s'\n" , nombre);
#endif
	if(dibujar != 0)
	{
		this->borrar_mapa();
		this->borrar_pan();
		x = pos.di_x(); y = pos.di_y();
		s = sin(SDAI_PI / 4); c = cos(SDAI_PI / 4);
		s1 = sin(angulo); c1 = cos(angulo);
		mapa->circulo((int)x , (int)y , radio , color%COLOR_INV);
		mapa->recta((int)x , (int)y , (int)(x + radio * c1) , (int)(y - radio * s1) , sentido%COLOR_INV);
		mapa->recta((int)(x - radio * c) , (int)(y - radio * s) , (int)(x + radio * c) , (int)(y + radio * s) , sentido%COLOR_INV);
		mapa->recta((int)(x - radio * c) , (int)(y + radio * s) , (int)(x + radio * c) , (int)(y - radio * s) , sentido%COLOR_INV);
		sprintf(msg_sale , "MFC%.3f %.3f %d %c" , x , y , radio , color%COLOR_INV);
		ini_m->di_sig()->enviar_figura();
		sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x + (radio / 2) * c1) , (y - (radio / 2) * s1) , (x + radio * c1) , (y - radio * s1) , sentido%COLOR_INV);
		ini_m->di_sig()->enviar_figura();
		sprintf(msg_sale , "MFT%.3f %.3f %c %s" , x - 9 , y + 6 , sentido%COLOR_INV , nombre);
		ini_m->di_sig()->enviar_figura();
		sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x - radio * c) , (y - radio * s) , (x + radio * c) , (y + radio * s) , sentido%COLOR_INV);
		ini_m->di_sig()->enviar_figura();
		sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x - radio * c) , (y + radio * s) , (x + radio * c) , (y - radio * s) , sentido%COLOR_INV);
		ini_m->di_sig()->enviar_figura();
	}
	sprintf(msg_sale , "RBR%s" , nombre);
	while(aux != NULL)
	{
		if(strcmp(nombre , aux->nombre) != 0)
			enviar_msg(aux->sock , &(aux->accesible));
		aux = aux->sig;
	}
	if(cerrar != 0) shutdown(sock , 2);
	if(ant != NULL) ant->sig = sig;
	if(sig != NULL) { sig->ant = ant; sig = NULL; }
	if(fin_r == this) fin_r = ant;
	fin = this; delete(fin);
}

int robot::aceptar_nuevos(void)
{
 int s , longitud , puerto , acceso = CIERTO;
 struct sockaddr_in cliente;
 fd_set test;
 robot *ns , *aux;
 struct timeval espera;
 struct linger opcion;
 int codigo = 0;

	FD_ZERO(&test);
	FD_SET(sock , &test);
	espera.tv_sec = 0;
	espera.tv_usec = 0;
	select(sock + 1 , SOCKET &test , NULL , NULL , &espera);
	if(FD_ISSET(sock , &test))
	{
#ifdef DEBUG
		printf("Peticion de conexion de un nuevo proceso robot\n");
		if(fichero == CIERTO) fprintf(accion , "Peticion de conexion de un nuevo proceso robot\n");
#endif
		longitud = sizeof(cliente);
		if((s = accept(sock , (struct sockaddr *) &cliente , &longitud)) < 0)
		{
			printf("\aERROR: no se puede aceptar la conexion del proceso robot.\n");
			if (fichero == CIERTO) fprintf(accion , "ERROR: no se puede aceptar la conexion del proceso robot.\n");
			return(-1);		    
		}
		if ( (ns = new robot) == NULL)
		{
			printf("\aERROR: no hay memoria suficiente para el proceso robot.\n");
			if(fichero == CIERTO) fprintf(accion , "ERROR: no hay memoria suficiente para el proceso robot.\n");
			close(s);
			return(-2);
		}
		if( (puerto = ns->crear()) < 0)
		{
			printf("\aERROR: No se puede crear el socket del proceso robot.\n");
			if(fichero == CIERTO) fprintf(accion , "ERROR: No se puede crear el socket del proceso robot.\n");
			close(s);
			return(-3);
		}
#ifdef DEBUG
		printf("Creo un nuevo socket para un proceso robot con el puerto %d\n" , puerto);
		if(fichero == CIERTO) fprintf(accion , "Creo un nuevo socket para un proceso robot con el puerto %d\n" , puerto);
#endif
		sprintf(msg_sale , "RNP%d" , puerto);
		if(enviar_msg(s , &acceso) < 0)
		{
			printf("\aERROR: No se puede enviar mensajes al proceso robot.\n");
			if (fichero == CIERTO) fprintf(accion , "ERROR: No se puede enviar mensajes al proceso robot.\n");
			ns->destruir(0 , 0);
			close(s);
			return(-4);
		}
#ifdef DEBUG
		printf("Envio al proceso robot '%s'\n" , msg_sale);
		if(fichero == CIERTO) fprintf(accion , "Envio al proceso robot '%s'\n" , msg_sale);
#endif
		FD_ZERO(&test);
		FD_SET(s , &test);
		espera.tv_sec = 3;
		espera.tv_usec = 0;
		select(s + 1 , SOCKET &test , NULL , NULL , &espera);
		if(FD_ISSET(s , &test))
		{
			if(recibir_msg(s , &acceso) < 0)
			{
				printf("\aERROR: No se puede recibir mensajes del proceso robot.\n");
				if (fichero == CIERTO) fprintf(accion , "ERROR: No se puede recibir mensajes del proceso robot.\n");
				ns->destruir(0 , 0);
				close(s);
				return(-5);
			}
		}
		else
		{
			printf("\aERROR: No hay contestacion del proceso robot.\n");
			if (fichero == CIERTO) fprintf(accion , "ERROR: No hay contestacion del proceso robot.\n");
			ns->destruir(0 , 0);
			close(s);
			return(-6);
		}
#ifdef DEBUG
		printf("Recibo del proceso robot '%s'\n" , msg_entra);
		if(fichero == CIERTO) fprintf(accion , "Recibo del proceso robot '%s'\n" , msg_entra);
#endif
		if( strcmp("RAP" , msg_entra) != 0)
		{
	        	ns->destruir(0 , 0);
			printf("\aERROR: falla la confirmacion del proceso robot.\n");
			if(fichero == CIERTO) fprintf(accion , "ERROR: falla la confirmacion del proceso robot.\n");
			close(s);
			return(-7);
		}
		close(s);
		s = accept(ns->di_sock() , (struct sockaddr *) &cliente , &longitud);
		opcion.l_linger = 5;
		opcion.l_onoff = 1;
		setsockopt(s , SOL_SOCKET , SO_LINGER , &opcion , sizeof(opcion));
#ifdef DEBUG
		printf("Socket %d de cierre diferido\n" , s);
		if(fichero == CIERTO) fprintf(accion , "Socket %d de cierre diferido\n" , s);
#endif
		close(ns->di_sock());
		ns->fijar_sock(s);
#ifdef DEBUG
		printf("Acepto un nuevo proceso robot en el socket %d\n" , s);
		if(fichero == CIERTO) fprintf(accion , "Acepto un nuevo proceso robot en el socket %d\n" , s);
#endif
		if((codigo = ns->obtener_datos()) != 0)
		{
			sprintf(msg_sale , "CDR%d" , codigo);
			enviar_msg(ns->sock , &acceso);
			ns->destruir(1 , 0);
			printf("\aERROR: hay un error en la obtencion de los datos del robot; error %d.\n" , codigo);
			if(fichero == CIERTO) fprintf(accion , "ERROR: hay un error en la obtencion de los datos del robot; error %d.\n" , codigo);
			return(-8);
		}
#ifdef DEBUG
		printf("Obtengo los datos del robot '%s'\n" , ns->nombre);
		if(fichero == CIERTO) fprintf(accion , "Obtengo los datos del robot '%s'\n" , ns->nombre);
#endif
		if((codigo = ns->comprobar_datos()) != 0)
		{
			sprintf(msg_sale , "CDR%d" , codigo);
			enviar_msg(ns->sock , &acceso);
			ns->destruir(1 , 0);
			printf("\aERROR: hay un error en los datos del robot; error %d.\n" , codigo);
			if(fichero == CIERTO) fprintf(accion , "ERROR: hay un error en los datos del robot; error %d.\n" , codigo);
			return(-9);
		}
		sprintf(msg_sale , "CDR%d" , codigo);
		if(enviar_msg(ns->sock , &acceso) < 0)
		{
			printf("\aERROR: No se puede enviar mensajes al proceso robot.\n");
			if (fichero == CIERTO) fprintf(accion , "ERROR: No se puede enviar mensajes al proceso robot.\n");
			ns->destruir(1 , 0);
			return(-10);
		}
#ifdef DEBUG
		printf("Compruebo los datos del robot '%s'\n" , ns->nombre);
		if(fichero == CIERTO) fprintf(accion , "Compruebo los datos del robot '%s'\n" , ns->nombre);
#endif
		aux = ini_r->sig;
		while(aux->sig != NULL)
		{
			sprintf(msg_sale , "RNR%s" , aux->nombre);
			if(enviar_msg(ns->sock , &acceso) < 0)
			{
				printf("\aERROR: No se puede enviar mensajes al proceso robot.\n");
				if (fichero == CIERTO) fprintf(accion , "ERROR: No se puede enviar mensajes al proceso robot.\n");
				ns->destruir(1 , 0);
				return(-11);
			}
			aux = aux->sig;
		}
#ifdef DEBUG
		printf("Notifico al nuevo robot '%s' la existencia de otros robots\n" , ns->nombre);
		if(fichero == CIERTO) fprintf(accion , "Notifico al nuevo robot '%s' la existencia de otros robots\n" , ns->nombre);
#endif
		aux = ini_r->sig;
		sprintf(msg_sale , "RNR%s" , ns->nombre);
		while(aux->sig != NULL)
		{
			enviar_msg(aux->sock , &(aux->accesible));
			aux = aux->sig;
		}
#ifdef DEBUG
		printf("Notifico a los robots la existencia de un nuevo robot '%s'\n" , ns->nombre);
		if(fichero == CIERTO) fprintf(accion , "Notifico a los robots la existencia de un nuevo robot '%s'\n" , ns->nombre);
#endif
		return(1);
	}
	return(0);
}

int robot::obtener_datos(void)
{
 float x , y;
 char codigo[4] = { '\0' , '\0' , '\0' , '\0' };
 sensor *sen;

	recibir_msg(sock , &accesible);
	sscanf(msg_entra , "DRN%s" , nombre);
#ifdef DEBUG
	printf("Leo el nombre del proceso robot '%s'\n" , nombre);
	if(fichero == CIERTO) fprintf(accion , "Leo el nombre del proceso robot '%s'\n" , nombre);
#endif
	recibir_msg(sock , &accesible);
	sscanf(msg_entra , "DRP%f %f %f" , &x , &y , &angulo);
	pos.fijar_punto(x , y); ult.fijar_punto(x , y); ult_angulo = angulo;
#ifdef DEBUG
	printf("Leo la posicion %d %d %d del proceso robot '%s'\n" , (int)pos.di_x() , (int)pos.di_y() , grados(angulo) , nombre);
	if(fichero == CIERTO) fprintf(accion , "Leo la posicion %d %d %d del proceso robot '%s'\n" , (int)pos.di_x() , (int)pos.di_y() , grados(angulo) , nombre);
#endif
	recibir_msg(sock , &accesible);
	sscanf(msg_entra , "DRA%d %d %c %c %c" , &radio , &ruedas , &color , &rastro , &sentido);
#ifdef DEBUG
	printf("Leo el aspecto %d %d %d %d %d del proceso robot '%s'\n" , radio , ruedas , color , rastro , sentido , nombre);
	if(fichero == CIERTO) fprintf(accion , "Leo el aspecto %d %d %d %d %d del proceso robot '%s'\n" , radio , ruedas , color  ,rastro , sentido , nombre);
#endif
	recibir_msg(sock , &accesible);
	while( strcmp(msg_entra , "DRF") != 0)
	{
#ifdef DEBUG
		printf("Leo el sensor '%s' del proceso robot '%s'\n" , msg_entra , nombre);
		if(fichero == CIERTO) fprintf(accion , "Leo el sensor '%s' del proceso robot '%s'\n" , msg_entra , nombre);
#endif
		strncpy(codigo , msg_entra , 3);
		if( strcmp(codigo , "DSL") == 0)
		{
			if( (sen = new laser) == NULL)
			{
#ifdef DEBUG
				printf("\aERROR: No hay memoria suficiente para el sensor laser.\n");
				if(fichero == CIERTO) fprintf(accion , "ERROR: No hay memoria suficiente para el sensor laser.\n");
#endif
				return(1);
			}
			else sen->crear(ini_s , fin_s , &msg_entra[3]);
		}
		else if( strcmp(codigo , "DSS") == 0)
		{
			if( (sen = new sonar) == NULL)
			{
#ifdef DEBUG
				printf("\aERROR: No hay memoria suficiente para el sensor sonar.\n");
				if(fichero == CIERTO) fprintf(accion , "ERROR: No hay memoria suficiente para el sensor sonar.\n");
#endif
				return(2);
			}
			else sen->crear(ini_s , fin_s , &msg_entra[3]);
		}
		else if( strcmp(codigo , "DSG") == 0)
		{
			if( (sen = new GPS) == NULL)
			{
#ifdef DEBUG
				printf("\aERROR: No hay memoria suficiente para el sensor GPS.\n");
				if(fichero == CIERTO) fprintf(accion , "ERROR: No hay memoria suficiente para el sensor GPS.\n");
#endif
				return(3);
			}
			else sen->crear(ini_s , fin_s , NULL);
		}
		else
		{
			printf("\aERROR: No entiendo los datos '%s' del proceso robot '%s'.\n" , msg_entra , nombre);
			if(fichero == CIERTO) fprintf(accion , "ERROR: No entiendo los datos '%s' del proceso robot '%s'.\n" , msg_entra , nombre);
			return(4);
		}
		recibir_msg(sock , &accesible);
	}
#ifdef DEBUG
	printf("Leo el fin '%s' de los datos del proceso robot '%s'\n" , msg_entra , nombre);
	if(fichero == CIERTO) fprintf(accion , "Leo el fin '%s' de los datos del proceso robot '%s'\n" , msg_entra , nombre);
#endif
	return(0);
}

void robot::responder_peticiones(void)
{
 char codigo[4] = { '\0' , '\0' , '\0' , '\0' };
 char destino[6] , mensaje[45];
 fd_set test;
 robot *aux = this , *fin , *des;
 struct timeval espera;

	while(aux != NULL)
	{
#ifdef DEBUG
		printf("Comprobando el proceso robot '%s'\n" , aux->nombre);
		if(fichero == CIERTO) fprintf(accion , "Comprobando el proceso robot '%s'\n" , aux->nombre);
#endif
		FD_ZERO(&test);
		FD_SET(aux->sock , &test);
		espera.tv_sec = 0;
		espera.tv_usec = 0;
		select(aux->sock + 1 , SOCKET &test , NULL , NULL , &espera);
		if(FD_ISSET(aux->sock , &test))
		{
			recibir_msg(aux->sock , &(aux->accesible));
#ifdef DEBUG
			printf("Recibo del proceso robot '%s': '%s'\n" , aux->nombre , msg_entra);
			if(fichero == CIERTO) fprintf(accion , "Recibo del proceso robot '%s': '%s'\n" , aux->nombre , msg_entra);
#endif
			strncpy(codigo , msg_entra , 3);
			if( strcmp(codigo , "ETS") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' lee sensores\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' lee sensores\n" , aux->nombre);
#endif
				aux->ini_s->explorar_sensores(aux);
			}
			else if( strcmp(codigo , "RCV") == 0)
			{
				sscanf(msg_entra , "RCV%f %f" , &aux->v1 , &aux->v2);
#ifdef DEBUG
				printf("El proceso robot '%s' cambia la velocidad %d %d\n" , aux->nombre , (int)aux->v1 , (int)aux->v2);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' cambia la velocidad %d %d\n" , aux->nombre , (int)aux->v1 , (int)aux->v2);
#endif
			}
			else if( strcmp(codigo , "REM") == 0)
			{
				sscanf(msg_entra , "REM%s %s" , destino , mensaje);
#ifdef DEBUG
				printf("El proceso robot '%s' envia el mensaje '%s' al robot '%s'\n" , aux->nombre , mensaje , destino);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' envia el mensaje '%s' al robot '%s'\n" , aux->nombre , mensaje , destino);
#endif
				des = ini_r->sig;
				while(des != NULL && strcmp(des->nombre , destino) != 0) des = des->sig;
				if(des != NULL)
				{
#ifdef DEBUG
					printf("El mensaje '%s' es enviado al robot '%s'\n" , msg_sale , des->nombre);
					if(fichero == CIERTO) fprintf(accion , "El mensaje '%s' es enviado al robot '%s'\n" , mensaje , des->nombre);
#endif
					sprintf(msg_sale , "MEM%s %s" , aux->nombre , mensaje);
					enviar_msg(des->sock , &(des->accesible));
				}
				else
				{
#ifdef DEBUG
					printf("\aERROR: No se encuentra al robot '%s'.\n" , destino);
					if(fichero == CIERTO) fprintf(accion , "ERROR: No se encuentra al robot '%s'.\n" , destino);
#endif
					sprintf(msg_sale , "FEM%s %s" , destino , mensaje);
					enviar_msg(aux->sock , &(aux->accesible));
				}
			}
			else if( strcmp(codigo , "RHP") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' se ha parado\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' se ha parado\n" , aux->nombre);
#endif
				aux->activo = FALSO;
			}
			else if( strcmp(codigo , "RHC") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' continua su ejecucion\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' continua su ejecucion\n" , aux->nombre);
#endif
				aux->activo = CIERTO;
			}
			else if( strcmp(codigo , "RMS") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' quiere dibujar los sensores\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' quiere dibujar los sensores\n" , aux->nombre);
#endif
				aux->dibu_sen = CIERTO;
			}
			else if( strcmp(codigo , "ROS") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' no quiere dibujar los sensores\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' no quiere dibujar los sensores\n" , aux->nombre);
#endif
				aux->dibu_sen = FALSO;
			}
			else if( strcmp(codigo , "RMR") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' quiere dibujar el rastro\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' quiere dibujar el rastro\n" , aux->nombre);
#endif
				aux->dibu_rastro = CIERTO;
			}
			else if( strcmp(codigo , "ROR") == 0)
			{
#ifdef DEBUG
				printf("El proceso robot '%s' no quiere dibujar el rastro\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' no quiere dibujar el rastro\n" , aux->nombre);
#endif
				aux->dibu_rastro = FALSO;
			}
			else if( strcmp(codigo , "RHT") == 0)
			{
				strcpy(msg_sale , "RPT");
				enviar_msg(aux->sock , &(aux->accesible));
				printf("El proceso robot '%s' quiere terminar\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El proceso robot '%s' quiere terminar\n" , aux->nombre);
				fin = aux; aux = aux->ant; fin->destruir(1 , 1);
				ventana->redibujar(1);
			}
		}
		else
		{
#ifdef DEBUG
			printf("No hay ninguna peticion del proceso robot '%s'\n" , aux->nombre);
			if(fichero == CIERTO) fprintf(accion , "No hay ninguna peticion del proceso robot '%s'\n" , aux->nombre);
#endif
		}
#ifdef DEBUG
		printf("Proceso robot '%s', accesible = %d\n" , aux->nombre , aux->accesible);
		if(fichero == CIERTO) fprintf(accion , "Proceso robot '%s', accesible = %d\n" , aux->nombre , aux->accesible);
#endif
		aux = aux->sig;
	}
}

int robot::escribir_lista(char *fich , int copia)
{
 FILE *rob;
 robot *aux = this;
 int i = 0;
 static int num = 1;
 char nombre[50] , version[10];

 	strcpy(nombre , fich);
	if(copia != 0)
	{
		sprintf(version , ".%d" , num);
		strcat(nombre , version);
		++num;
	}
	if ( (rob = fopen(nombre , "wt")) == NULL)
	{
		printf("\aERROR: No puedo abrir el fichero '%s'.\n" , nombre);
		if(fichero == CIERTO) fprintf(accion , "ERROR: No puedo abrir el fichero '%s'.\n" , nombre);
		return (-1);
	}
#ifdef DEBUG
	printf("Abro el fichero '%s' para escribir los procesos robot\n" , nombre);
	if(fichero == CIERTO) fprintf(accion , "Abro el fichero '%s' para escribir los procesos robot\n" , nombre);
#endif
	while(aux != NULL)
	{
		fprintf(rob , "Proceso robot '%s' socket %d\n" , aux->nombre , aux->sock);
		aux = aux->sig;
     	i++;
	}
#ifdef DEBUG
	printf("Escribo los %d procesos robot del mundo\n" , i);
	if(fichero == CIERTO) fprintf(accion , "Escribo los %d procesos robot del mundo\n" , i);
#endif
	fclose(rob);
	return(0);
}

void robot::terminar(void)
{
 robot *aux = this;

	sprintf(msg_sale , "RFY");
 	while(aux != NULL)
	{
		enviar_msg(aux->sock , &(aux->accesible));
		aux = aux->sig;
#ifdef DEBUG
		printf("Mando al proceso robot '%s' '%s'\n" , aux->nombre , msg_sale);
		if(fichero == CIERTO) fprintf(accion , "Mando al proceso robot '%s' '%s'\n" , aux->nombre , msg_sale);
#endif
	}
}

void robot::dibujar_mapa(void)
{
 float x , y , s , c;

	x = pos.di_x(); y = pos.di_y();
	s = sin(angulo); c = cos(angulo);
#ifdef DEBUG
	printf("Dibujo el robot '%s' en el mapa en X=%d Y=%d\n" , nombre , (int)x , (int)y);
	if(fichero == CIERTO) fprintf(accion , "Dibujo el robot '%s' en el mapa en X=%d Y=%d\n" , nombre , (int)x , (int)y);
#endif
	mapa->circulo((int)x , (int)y , radio , color);
	mapa->recta((int)x , (int)y , (int)(x + radio * c) , (int)(y - radio * s) , sentido);
}

void robot::dibujar_pan(void)
{
 float x , y , s , c;

	x = pos.di_x(); y = pos.di_y();
	s = sin(angulo); c = cos(angulo);
#ifdef DEBUG
	printf("Dibujo el robot '%s' en la pantalla en X=%d Y=%d\n" , nombre , (int)x , (int)y);
	if(fichero == CIERTO) fprintf(accion , "Dibujo el robot '%s' en la pantalla en X=%d Y=%d\n" , nombre , (int)x , (int)y);
#endif
	sprintf(msg_sale , "MFC%.3f %.3f %d %c" , x , y , radio , color);
	ini_m->di_sig()->enviar_figura();
	sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x + (radio / 2) * c) , (y - (radio / 2) * s) , (x + radio * c) , (y - radio * s) , sentido);
	ini_m->di_sig()->enviar_figura();
	sprintf(msg_sale , "MFT%.3f %.3f %c %s" , x - 9 , y + 6 , sentido , nombre);
	ini_m->di_sig()->enviar_figura();
	ult.fijar_punto(x , y);
	ult_angulo = angulo;
}

void robot::borrar_mapa(void)
{
 float ang , x , y , s , c , x1 , y1;

	x = pos.di_x(); y = pos.di_y();
	s = sin(angulo); c = cos(angulo);
#ifdef DEBUG
	printf("Borro el robot '%s' del mapa en X=%d Y=%d\n" , nombre , (int)x , (int)y);
	if(fichero == CIERTO) fprintf(accion , "Borro el robot '%s' del mapa en X=%d Y=%d\n" , nombre , (int)x , (int)y);
#endif
	mapa->circulo((int)x , (int)y , radio , 0);
	mapa->recta((int)x , (int)y , (int)(x + radio * c) , (int)(y - radio * s) , 0);
	if(dibu_rastro == CIERTO)
	{
		ang = angulo(angulo + SDAI_PI);
		s = sin(ang); c = cos(ang);
		mapa->recta((int)x , (int)y , (int)(x + radio * c) , (int)(y - radio * s) , rastro);
	}
}

void robot::borrar_pan(void)
{
 float ang , x , y , s , c;

#ifdef DEBUG
	printf("Borro el robot '%s' de la pantalla en X=%d Y=%d\n" , nombre , (int)x , (int)y);
	if(fichero == CIERTO) fprintf(accion , "Borro el robot '%s' de la pantalla en X=%d Y=%d\n" , nombre , (int)x , (int)y);
#endif
	x = ult.di_x(); y = ult.di_y();
	s = sin(ult_angulo); c = cos(ult_angulo);
	sprintf(msg_sale , "MFC%.3f %.3f %d %c" , x , y , radio , 16);
	ini_m->di_sig()->enviar_figura();
	sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x + (radio / 2) * c) , (y - (radio / 2) * s) , (x + radio * c) , (y - radio * s) , 16);
	ini_m->di_sig()->enviar_figura();
	sprintf(msg_sale , "MFT%.3f %.3f %c %s" , x - 9 , y + 6 , 16 , nombre);
	ini_m->di_sig()->enviar_figura();
	if(dibu_rastro == CIERTO)
	{
		ang = angulo(ult_angulo + SDAI_PI);
		s = sin(ang); c = cos(ang);
		sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , x , y , (x + radio * c) , (y - radio * s) , rastro);
		ini_m->di_sig()->enviar_figura();
	}
}

int robot::comprobar_datos(void)
{
 robot *aux = ini_r->sig;

	if( ((int)pos.di_x() + radio + 2) > planox || ((int)pos.di_x() - radio - 2) < 0 ) return(5);
	if( ((int)pos.di_y() + radio + 2) > planoy || ((int)pos.di_y() - radio - 2) < 0 ) return(6);
	while(aux->sig != NULL)
	{
		if(strcmp(nombre , aux->nombre) == 0) return(7);
		aux = aux->sig;
	}
	if( comprobar((int)pos.di_x() , (int)pos.di_y()) ) return(8);
#ifdef DEBUG
	printf("Compruebo los datos del robot '%s'\n" , nombre);
	if(fichero == CIERTO) fprintf(accion , "Compruebo los datos del robot '%s'\n" , nombre);
#endif
	return(0);
}

int robot::comprobar(int xc , int yc)
{
 register int x = 0;
 register int y = radio;
 long a = radio , a2 = a * a , dosa2 = 2 * a2;
 long d = a2 - a2 * a + a2 / 4L , dx = 0 , dy = dosa2 * a;
 unsigned char color = 0;

	while(dx < dy && color < COLOR_INV)
	{
		color |= mapa->di_punto(xc + x , yc + y);
		color |= mapa->di_punto(xc - x , yc + y);
		color |= mapa->di_punto(xc + x , yc - y);
		color |= mapa->di_punto(xc - x , yc - y);
		if(d > 0L) { --y; dy-=dosa2; d-=dy; }
		++x; dx+=dosa2; d+=a2 + dx;
	}
	d+=(-(dx + dy)) / 2L;
	while(y >= 0 && color < COLOR_INV)
	{
		color |= mapa->di_punto(xc + x , yc + y);
		color |= mapa->di_punto(xc - x , yc + y);
		color |= mapa->di_punto(xc + x , yc - y);
		color |= mapa->di_punto(xc - x , yc - y);
		if(d < 0L) { ++x; dx+=dosa2; d+=dx; }
		--y; dy-=dosa2; d+=a2 - dy;
	}
#ifdef DEBUG
	printf("Compruebo el robot X=%d Y=%d radio=%d en el mapa color %d\n" , (int)pos.di_x() , (int)pos.di_y() , radio , color);
	if(fichero == CIERTO) fprintf(accion , "Compruebo el robot X=%d Y=%d radio=%d en el mapa color %d\n" , (int)pos.di_x() , (int)pos.di_y() , radio , color);
#endif
	if(color < COLOR_INV) return(0); else return(1);
}

void robot::enviar_robots(int socket)
{
 robot *aux = this;

 	while(aux != NULL)
	{
		aux->enviar_robot(socket);
		aux = aux->sig;
	}
}

void robot::enviar_robot(int socket)
{
 float x , y , s , c;

	x = ult.di_x(); y = ult.di_y();
	s = sin(ult_angulo); c = cos(ult_angulo);
#ifdef DEBUG
	printf("Envio el robot '%s' en X=%d Y=%d\n" , nombre , (int)x , (int)y);
	if(fichero == CIERTO) fprintf(accion , "Envio el robot '%s' en X=%d Y=%d\n" , nombre , (int)x , (int)y);
#endif
	sprintf(msg_sale , "MFC%.3f %.3f %d %c" , x , y , radio , color);
	enviar_msg(socket , &accesible);
	sprintf(msg_sale , "MFR%.3f %.3f %.3f %.3f %c" , (x + (radio / 2) * c) , (y - (radio / 2) * s) , (x + radio * c) , (y - radio * s) , sentido);
	enviar_msg(socket , &accesible);
	sprintf(msg_sale , "MFT%.3f %.3f %c %s" , x - 9 , y + 6 , sentido , nombre);
	enviar_msg(socket , &accesible);
}

void robot::mover(void)
{
 robot *aux = this;
 float v , w , ang , x , y;

	while(aux != NULL)
	{
		if(aux->activo == CIERTO)
		{
#ifdef DEBUG
			printf("Moviendo al proceso robot '%s'\n" , aux->nombre);
			if(fichero == CIERTO) fprintf(accion , "Moviendo al proceso robot '%s'\n" , aux->nombre);
#endif
			w = (aux->v2 - aux->v1) / ruedas;
			ang = angulo(aux->angulo + w * PASO);
			v = (aux->v1 + aux->v2) / 2;
			x = aux->pos.di_x() + ((v * PASO) * cos(ang));
			y = aux->pos.di_y() - ((v * PASO) * sin(ang));
			aux->borrar_mapa();
			if(aux->comprobar((int)x , (int)y))
			{
#ifdef DEBUG
				printf("El robot '%s' se ha chocado\n" , aux->nombre);
				if(fichero == CIERTO) fprintf(accion , "El robot '%s' se ha chocado\n" , aux->nombre);
#endif
				sprintf(msg_sale , "RTC");
				enviar_msg(aux->sock , &(aux->accesible));
			}
			else
			{
				if(dib == 0) aux->borrar_pan();
				aux->pos.fijar_punto(x , y);
				aux->angulo = ang;
				if(dib == 0) aux->dibujar_pan();
			}
			aux->dibujar_mapa();
#ifdef DEBUG
			printf("Robot '%s' x=%d y=%d angulo=%d\n" , aux->nombre , (int)aux->pos.di_x() , (int)aux->pos.di_y() , grados(aux->angulo));
			if(fichero == CIERTO) fprintf(accion , "Robot '%s' x=%d y=%d angulo=%d\n" , aux->nombre , (int)aux->pos.di_x() , (int)aux->pos.di_y() , grados(aux->angulo));
#endif
		}
		aux = aux->sig;
	}
}

char* robot::di_datos(void)
{
 static char linea[100];

	sprintf(linea , "Robot %s    x =%5d   y =%5d   sentido =%4d   Vizq =%5.1f   Vder =%5.1f   %s" , nombre , (int)pos.di_x() , (int)pos.di_y() , grados(angulo) , v1 , v2 , (activo == CIERTO) ? "Activo" : "Parado");
	return(linea);
}

void robot::parar_robots(void)
{
 robot *aux = this;

	strcpy(msg_sale , "RPY");
	while(aux != NULL)
	{
		enviar_msg(aux->sock , &(aux->accesible));
#ifdef DEBUG
		printf("Paro al proceso robot '%s'\n" , aux->nombre);
		if(fichero == CIERTO) fprintf(accion , "Paro al proceso robot '%s'\n" , aux->nombre);
#endif
		aux->activo = FALSO;
		aux = aux->sig;
	}
}

void robot::continuar_robots(void)
{
 robot *aux = this;

	strcpy(msg_sale , "RCY");
	while(aux != NULL)
	{
		enviar_msg(aux->sock , &(aux->accesible));
#ifdef DEBUG
		printf("Mando continuar al proceso robot '%s'\n" , aux->nombre);
		if(fichero == CIERTO) fprintf(accion , "Mando continuar al proceso robot '%s'\n" , aux->nombre);
#endif
		aux->activo = CIERTO;
		aux = aux->sig;
	}
}
