/*       SENSOR              Ignacio  Merino  Lagunilla                       */

#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "global.hpp"
#include "sensor.hpp"

//#define 	DEBUG


void sensor::explorar_sensores(robot *rob)
{
 sensor *aux = this;
 int i = 0;

	strcpy(msg_sale , "ETS");
	rob->enviar_msg(rob->di_sock());
	while(aux != NULL)
	{
		aux->explorar(rob , aux->salida);
		aux = aux->sig;
		++i;
	}
#ifdef DEBUG
	printf("Exploro los %d sensores del robot '%s'\n" , i , rob->di_nombre());
	if(fichero == CIERTO) fprintf(accion , "Exploro los %d sensores del robot '%s'\n" , i , rob->di_nombre());
#endif
}

void laser::crear(sensor *&ini , sensor *&fin , int dis , float ang , float an , int ran)
{
	if(ini == NULL)
	{
		ini = fin = this;
	}
	else
	{
		ant = fin;
		fin->fijar_sig(this);
		fin = this;
	}
	pos.fijar_punto(dis , ang);
	rango = ran;
	angulo = an;
#ifdef DEBUG
	printf("Creo el sensor laser dis = %d ang = %d orientacion = %d rango = %d\n" , dis , grados(ang) , grados(an) , ran);
	if(fichero == CIERTO) fprintf(accion , "Creo el sensor laser dis = %d ang = %d orientacion = %d rango = %d\n" , dis , grados(ang) , grados(an) , ran);
#endif
}

void laser::explorar(robot *rob , float *salida)
{
	rob->recibir_msg(rob->di_sock() , "RSL" , 0);
	sscanf(msg_entra , "RSL%f %f" , &salida[0] , &salida[1]);
#ifdef DEBUG
	printf("'%s' datos sensor laser distancia = %d color = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1]);
	if(fichero == CIERTO) fprintf(accion , "'%s' datos sensor laser distancia = %d color = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1]);
#endif
}

void laser::enviar_datos(robot *rob)
{
	sprintf(msg_sale , "DSL%d %f %f %d" , (int)pos.di_dis() , pos.di_ang() , angulo , rango);
	rob->enviar_msg(rob->di_sock());
#ifdef DEBUG
	printf("Envio los datos del sensor laser %d %d %d %d\n" , (int)pos.di_dis() , grados( pos.di_ang() ) , grados(angulo) ,rango);
	if(fichero == CIERTO) fprintf(accion , "Envio los datos del sensor laser %d %d %d %d\n" , (int)pos.di_dis() , grados( pos.di_ang() ) , grados(angulo) , rango);
#endif
}

char* laser::di_datos(void)
{
 static char linea[100];

	if((int)salida[0] < rango) sprintf(linea , "Sensor laser           distancia =%3d     color =%4d" , (int)salida[0] , (int)salida[1]);
	else sprintf(linea , "Sensor laser           Ningun obstaculo               ");
	return(linea);
}

void sonar::crear(sensor *&ini , sensor *&fin , int dis , float ang , float an , int ran , float an_sec , float interv , char tip)
{
	if(ini == NULL)
	{
		ini = fin = this;
	}
	else
	{
		ant = fin;
		fin->fijar_sig(this);
		fin = this;
	}
	pos.fijar_punto(dis , ang);
	rango = ran;
	angulo = an;
	ang_sec = an_sec;
	intervalo = interv;
	tipo = tip;
#ifdef DEBUG
	printf("Creo el sensor sonar dis = %d ang = %d orientacion = %d rango = %d ang_sec = %d interv = %d tipo = %d\n" , dis , grados(ang) , grados(angulo) , rango , grados(ang_sec) , grados(intervalo) , tipo);
	if(fichero == CIERTO) fprintf(accion , "Creo el sensor sonar dis = %d ang = %d orientacion = %d rango = %d ang_sec = %d interv = %d tipo = %d\n" , dis , grados(ang) , grados(angulo) , rango , grados(ang_sec) , grados(intervalo) , tipo);
#endif
}

void sonar::explorar(robot *rob , float *salida)
{
	rob->recibir_msg(rob->di_sock() , "RSS" , 0);
	sscanf(msg_entra,"RSS%f %f" , &salida[0] , &salida[1]);
#ifdef DEBUG
	printf("'%s' datos sensor sonar distancia = %d color = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1]);
	if(fichero == CIERTO) fprintf(accion , "'%s' datos sensor sonar distancia = %d color = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1]);
#endif
}

void sonar::enviar_datos(robot *rob)
{
	sprintf(msg_sale , "DSS%d %.3f %.3f %d %.3f %.3f %c" , (int)pos.di_dis() , pos.di_ang() , angulo , rango , ang_sec , intervalo , tipo);
	rob->enviar_msg(rob->di_sock());
#ifdef DEBUG
	printf("Envio los datos del sensor sonar %d %d %d %d %d %d %d\n" , (int)pos.di_dis() , grados( pos.di_ang() ) , grados(angulo) , rango , grados(ang_sec) , grados(intervalo) , (int)tipo);
	if(fichero == CIERTO) fprintf(accion , "Envio los datos del sensor sonar %d %d %d %d %d %d %d\n" , (int)pos.di_dis() , grados( pos.di_ang() ) , grados(angulo) , rango , grados(ang_sec) , grados(intervalo) , (int)tipo);
#endif
}

char* sonar::di_datos(void)
{
 static char linea[100];

	if((int)salida[0] < rango) sprintf(linea , "Sensor sonar  %6s   distancia =%3d     color =%4d" , ((tipo == 1) ? "minimo" : ((tipo == 2) ? "media" : "maximo")) , (int)salida[0] , (int)salida[1]);
	else sprintf(linea , "Sensor sonar  %6s   Ningun obstaculo               " , ((tipo == 1) ? "minimo" : ((tipo == 2) ? "media" : "maximo")));
	return(linea);
}

void GPS::crear(sensor *&ini , sensor *&fin)
{
	if(ini == NULL)
	{
		ini = fin = this;
	}
	else
	{
		ant = fin;
		fin->fijar_sig(this);
		fin = this;
	}
#ifdef DEBUG
	printf("Creo el sensor GPS\n");
	if(fichero == CIERTO) fprintf(accion , "Creo el sensor GPS\n");
#endif
}

void GPS::explorar(robot *rob , float *salida)
{
	rob->recibir_msg(rob->di_sock() , "RSG" , 0);
	sscanf(msg_entra , "RSG%f %f %f" , &salida[0] , &salida[1] , &salida[2]);
	rob->di_pos()->fijar_punto(salida[0] , salida[1]);
	rob->fijar_angulo(salida[2]);
#ifdef DEBUG
	printf("'%s' datos sensor GPS x = %d y = %d sentido = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1] , grados( salida[2] ));
	if(fichero == CIERTO) fprintf(accion , "'%s' datos sensor GPS x = %d y = %d sentido = %d\n" , rob->di_nombre() , (int)salida[0] , (int)salida[1] , grados( salida[2] ));
#endif
}

void GPS::enviar_datos(robot *rob)
{
	sprintf(msg_sale , "DSG");
	rob->enviar_msg(rob->di_sock());
#ifdef DEBUG
	printf("Envio los datos del sensor GPS\n");
	if(fichero == CIERTO) fprintf(accion , "Envio los datos del sensor GPS\n");
#endif
}

char* GPS::di_datos(void)
{
 static char linea[100];

	sprintf(linea,"Sensor GPS        x =%5d     y =%5d     sentido =%4d" , (int)salida[0] , (int)salida[1] , grados( salida[2] ));
	return(linea);
}
