/******************************************************************************\
*                                                                              *
*       SENSORES            Ignacio  Merino  Lagunilla                         *
*                                                                              *
*       Funciones para definir los distintos sensores de los robots.           *
*                                                                              *
*       22/febrero/1995                                                        *
\******************************************************************************/

#ifndef _SENSORES_
#define _SENSORES_

#include <stdio.h>

#include "coord.hpp"

class robot;

class sensor{
	protected:
		punto_pol pos;							// posicion del sensor
		float *salida;							// datos de salida del sensor
		sensor *sig;
		sensor *ant;
	public:
		inline sensor(void) { sig = ant = NULL; salida = NULL; }
		virtual ~sensor(void) { }
		inline sensor* di_sig(void) { return(sig); }
		inline void fijar_sig(sensor *s) { sig = s; }
		inline sensor* di_ant(void) { return(ant); }
		inline void fijar_ant(sensor *a) { ant = a; }
		inline float* di_salida(void) { return(salida); }
		void explorar_sensores(robot *rob);
		virtual void explorar(robot *rob , float *salida) = 0;
		virtual void enviar_datos(robot *rob) = 0;
		virtual char* di_datos(void) = 0;
};

class laser:public sensor{
	private:
		float angulo;						// angulo de orientacion
		int rango;						// alcance del sensor
	public:
		inline laser(void) { salida = new float[2]; salida[0] = 0; salida[1] = 0; }
		inline ~laser(void) { if(sig != NULL) delete(sig); if(salida != NULL) delete(salida);  }
		void crear(sensor *&ini , sensor *&fin , int dis , float ang , float angulo , int rango);
		void explorar(robot *rob , float *salida);
		void enviar_datos(robot *rob);
		char* di_datos(void);
};

class sonar:public sensor{
	private:
		float angulo;						// angulo de orientacion
		int rango;						// alcance del sensor
		float ang_sec;						// angulo del sector
		float intervalo;					// intervalo exploracion laser
		char tipo;						// 1=minimo 2=media 3=maximo
	public:
		inline sonar(void) { salida = new float[2]; salida[0] = 0; salida[1] = 0; }
		inline ~sonar(void) { if(sig != NULL) delete(sig); if(salida != NULL) delete(salida); }
		void crear(sensor *&ini , sensor *&fin , int dis , float ang , float angulo , int rango , float an_sec , float inter , char tipo);
		void explorar(robot *rob , float *salida);
		void enviar_datos(robot *rob);
		char* di_datos(void);
};

class GPS:public sensor{
	private:
	public:
		inline GPS(void) { salida = new float[3]; salida[0] = 0; salida[1] = 0; salida[2] = 0; }
		inline ~GPS(void) { if(sig != NULL) delete(sig); if(salida != NULL) delete(salida); }
		void crear(sensor *&ini , sensor *&fin);
		void explorar(robot *rob , float *salida);
		void enviar_datos(robot *rob);
		char* di_datos(void);
};

#endif
